﻿package gamespecific
{
    import adobe.utils.CustomActions;
    import flash.events.Event;
    import flash.geom.Point;
    import meyermike.core.GameObjectHolder;
    import meyermike.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Cave
    {
        // how many planes we can handle at a time
        private const NUM_PLANES:uint = 12;

        // how much space we need between them (based on the art)
        private const DISTANCE_BETWEEN_PLANES:Number = 20;

        // where the first plane goes
        private const PLANE_START_Z:Number = 0;

        private var _runSpeed:Number = 0;
        public function get RunSpeed():Number { return _runSpeed; }
        public function set RunSpeed(value:Number):void { _runSpeed = value; }

        // where the planes of the cave will be put
        private var mCaveLayer:GameObjectHolder;
        private var mObstacleLayer:GameObjectHolder;

        private var _obstacleLevel:Number = 0;
        public function get ObstacleLevel():Number { return _obstacleLevel; }
        public function set ObstacleLevel(value:Number):void { _obstacleLevel=value; }

        private var _vanishingPoint:Point = new Point(0, 0);
        public function get VanishingPoint():Point { return _vanishingPoint; }
        private var _previousVanishingPoint:Point = new Point(0, 0);
        private var _targetVanishingPoint:Point = new Point(0, 0);

        private var mDistanceToNextVanishingPoint:Number = 0;

        private static const DISTANCE_BETWEEN_TURNS:Number = 250.0;

        private static const MAX_VANISHING_POINT_OFFSET:Number = 100;

        private var _caveBuildingEnabled:Boolean = true;
        public function get CaveBuildingEnabled():Boolean { return _caveBuildingEnabled; }
        public function set CaveBuildingEnabled(value:Boolean):void { _caveBuildingEnabled = value; }

        private var _numFoesSpawned:Number = 0;
        public function get NumFoesSpawned():Number { return _numFoesSpawned; }
        public function set NumFoesSpawned(value:Number):void { _numFoesSpawned = value; }

        private var _numFoesDefeated:Number = 0;
        public function get NumFoesDefeated():Number { return _numFoesDefeated; }
        public function set NumFoesDefeated(value:Number):void { _numFoesDefeated = value; }

        // how likely we are to spawn an enemy when we spawn something
        private var mChanceForEnemy:Number = 0.125;

        /**
         * set up
         */
        public function Init(caveLayer:GameObjectHolder, obstacleLayer:GameObjectHolder):void
        {
            mCaveLayer = caveLayer;
            mObstacleLayer = obstacleLayer;
            CaveBuildingEnabled = true;
            NumFoesSpawned = 0;
            NumFoesDefeated = 0;
            AddInitialPlanes();

            var theta:Number = Math.random() * 2.0 * Math.PI;
            var r:Number = Math.random() * MAX_VANISHING_POINT_OFFSET;
            VanishingPoint.x = gamespecific.Global.ScreenCenter.x + (r * Math.cos(theta) );
            VanishingPoint.y = gamespecific.Global.ScreenCenter.y + (r * Math.sin(theta) );
        }

        /**
         * add the planes that are viisble at the start of the game
         */
        private function AddInitialPlanes():void
        {
            var z:Number = PLANE_START_Z;
            for (var i:uint = 0; i < NUM_PLANES; ++i)
            {
                SpawnPlane(z);
                z += DISTANCE_BETWEEN_PLANES;
            }
        }


        public function Update(elapsedTime:Number):void
        {
            // Spawn planes when needed
            var numPlanes:uint = mCaveLayer.Objects.length;
            while (CaveBuildingEnabled && (NUM_PLANES > numPlanes) )
            {
                var z:Number = PLANE_START_Z;

                if (numPlanes > 0)
                {
                    var plane:Plane = mCaveLayer.Objects[numPlanes - 1];
                    z = plane.z + DISTANCE_BETWEEN_PLANES;
                }

                SpawnObstacles(z);
                SpawnPlane(z);
                ++numPlanes;
            }

            // vanishing point wiggles around to make it look like a curvy tunnel
            mDistanceToNextVanishingPoint -= RunSpeed*elapsedTime;

            if (mDistanceToNextVanishingPoint <= 0)
            {
                _previousVanishingPoint.x = VanishingPoint.x;
                _previousVanishingPoint.y = VanishingPoint.y;

                VanishingPoint.x = _targetVanishingPoint.x;
                VanishingPoint.y = _targetVanishingPoint.y;
                mDistanceToNextVanishingPoint = DISTANCE_BETWEEN_TURNS;

                var theta:Number = Math.random() * 2.0 * Math.PI;
                var r:Number = Math.random() * MAX_VANISHING_POINT_OFFSET;
                _targetVanishingPoint.x = gamespecific.Global.ScreenCenter.x + (r * Math.cos(theta));
                _targetVanishingPoint.y = gamespecific.Global.ScreenCenter.y + (r * Math.sin(theta));
            }
            else
            {
                var t:Number = mDistanceToNextVanishingPoint / DISTANCE_BETWEEN_TURNS;
                VanishingPoint.x = MathUtil.Lerp(_targetVanishingPoint.x, _previousVanishingPoint.x, t);
                VanishingPoint.y = MathUtil.Lerp(_targetVanishingPoint.y, _previousVanishingPoint.y, t);
            }
        }

        private function SpawnPlane(z:Number):void
        {
            var plane:Plane = mCaveLayer.Add(Plane) as Plane;
            plane.cave = this;
            plane.parent.setChildIndex(plane, 0); // push it to the "back"
            plane.z = z;
        }

        /**
         * Add obstacles based on the
         */
        private function SpawnObstacles(z:Number):void
        {
            var n:Number = ObstacleLevel;
            n -= Math.random();
            while (n > 0)
            {
                if (Math.random() < mChanceForEnemy)
                {
                    SpawnEnemy(z);
                }
                else
                {
                    SpawnObstacle(z);
                }
                n -= Math.random();
            }
        }

        /**
         * add a single obstacle now that we know we want one
         */
        private function SpawnObstacle(z:Number):void
        {
            var obstacle:Obstacle = mObstacleLayer.Add(Obstacle) as Obstacle;
            obstacle.cave = this;
            obstacle.parent.setChildIndex(obstacle, 0); // push it to the "back"
            obstacle.z = z;
        }

        /**
         * add a single enemy
         */
        private function SpawnEnemy(z:Number):void
        {
            var enemy:Enemy = mObstacleLayer.Add(Enemy) as Enemy;
            enemy.cave = this;
            enemy.parent.setChildIndex(enemy, 0); // push it to the "back"
            enemy.z = z;
            ++NumFoesSpawned;
            if (!enemy.hasEventListener(Enemy. DYING))
            {
                enemy.addEventListener(gamespecific.Enemy.DYING, EnemyDying);
            }
        }

        /**
         *
         */
        private function EnemyDying(e:Event):void
        {
            ++NumFoesDefeated;
        }

        /**
         * clean up
         */
        public function Shutdown():void
        {
            mCaveLayer = null;
            mObstacleLayer = null;
            mDistanceToNextVanishingPoint = 0;
        }
    }

}
