﻿package gamespecific
{
    import flash.geom.ColorTransform;
    import flash.geom.Point;
    import meyermike.constants.Colors;
    import meyermike.core.GameObject;
    import meyermike.Util;
    import meyermike.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class CaveObject extends GameObject
    {
        private var _z:Number = 0;
        public function get z():Number { return _z; }
        public function set z(value:Number):void { _z = value; }

        private var _cave:Cave;
        public function get cave():Cave { return _cave; }
        public function set cave(value:Cave):void { _cave = value; }

        /// for hacked up frustum culling
        private static const NEAR_PLANE:Number = -75;
        private static const FAR_PLANE:Number = 1000;

        private static const FADE_RATE:Number = 3.0;

        private static var REAL_POS:Point;

        protected var mXFlipped:Boolean;
        protected var mYFlipped:Boolean;

        protected var mNearRed:Number    = 0.537;
        protected var mFarRed:Number     = 0;
        protected var mNearGreen:Number  = 0.396;
        protected var mFarGreen:Number   = 0;
        protected var mNearBlue:Number   = 0.267;
        protected var mFarBlue:Number    = 0;

        /**
         * initial set up for cave objects
         */
        public override function Init():void
        {
            super.Init();

            if (null == REAL_POS)
            {
                REAL_POS = new Point(0.5 * meyermike.Global.StageWidth, 0.5 * meyermike.Global.StageHeight);
            }

            this.PhysicsEnabled = false;

            mXFlipped = Util.RandomBoolean();
            mYFlipped = Util.RandomBoolean();

            alpha = 0.0;
        }

        /**
         *
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            var scale:Number = PerspectiveUtil.GetScaleFromZ(this.z);

            this.visible = (this.z <= FAR_PLANE) && (this.z >= NEAR_PLANE);

            this.x = MathUtil.Lerp( cave.VanishingPoint.x, REAL_POS.x, scale );
            this.y = MathUtil.Lerp( cave.VanishingPoint.y, REAL_POS.y, scale );

            // alpha
            if (z < 0)
            {
                alpha = 1.0 - (z / NEAR_PLANE);
            }
            else
            {
                if (1.0 >= alpha)
                {
                    alpha += FADE_RATE * elapsedTime;
                }
            }

            // color
            var lerpValue:Number = scale;
            var colorTransform:ColorTransform = this.transform.colorTransform;
            colorTransform.redMultiplier = MathUtil.Lerp(mFarRed, mNearRed, lerpValue);
            colorTransform.greenMultiplier = MathUtil.Lerp(mFarGreen, mNearGreen, lerpValue);
            colorTransform.blueMultiplier = MathUtil.Lerp(mFarBlue, mNearBlue, lerpValue);
            this.transform.colorTransform  = colorTransform;

            this.scaleX = mXFlipped ? -scale : scale;
            this.scaleY = mYFlipped ? -scale : scale;
        }

        public override function Integrate(elapsedTime:Number):void
        {
            super.Integrate(elapsedTime);
            this.z -= cave.RunSpeed * elapsedTime;
        }

        /**
         * tells when it has scrolled away far enough that we don't have to worry about it anymore
         */
        public override function IsAlive():Boolean
        {
            return z > NEAR_PLANE;
        }

        /**
         * clean up
         */
        public override function Shutdown():void
        {
            super.Shutdown();
            mXFlipped = false;
            mYFlipped = false;
        }
    }

}
