﻿package gamespecific
{
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import flash.display.PixelSnapping;
    import flash.events.Event;
    import meyermike.util.MathUtil;


    /**
     * ...
     * @author Mike Meyer
     */
    public class Enemy extends CaveObject
    {
        public static const DYING : String = "dying";

        [Embed(source='assets/enemy.png')]
        private static const AliveImageClass:Class;
        private var mAliveImage:Sprite;

        private static const FADE_RATE:Number = 4.0;
        private var _dying:Boolean = false;
        public function get Dying():Boolean { return _dying; }
        public function set Dying(value:Boolean):void
        {
            if ( (!_dying) && (value) )
            {
                this.dispatchEvent(new Event(DYING));
                mFarRed = 1.0;
            }

            _dying = value;
        }

        public override function Init():void
        {
            super.Init();
            CreateGraphics();

            mNearRed   = 0.5;
            mFarRed   = 0.0;
            mNearGreen   = 0.75;
            mNearBlue   = 1.0;
            mFarBlue    = 0.4;
            Dying = false;
        }

        protected function CreateGraphics():void
        {
            if (null == mAliveImage)
            {
                var image:Bitmap = new AliveImageClass();
                image.smoothing = false;
                image.pixelSnapping = PixelSnapping.ALWAYS;
                image.x = -0.5 * image.width;
                image.y = -image.height;

                mAliveImage = new Sprite();
                mAliveImage.addChild(image);
                this.addChild(mAliveImage);
            }

            var theta:Number = Math.PI * 2.0 * Math.random();
            mAliveImage.x = (Global.TunnelRadius * Math.cos(theta));
            mAliveImage.y = (Global.TunnelRadius * Math.sin(theta));
            mAliveImage.rotation = -90.0 + (theta * 180.0 / Math.PI);
            mAliveImage.scaleX = (Math.random() < 0.75) ? -1.0 : 1.0;
        }

        public override function Update(elapsedTime:Number):void
        {
            if (Dying)
            {
                alpha -= FADE_RATE * elapsedTime;
                z += 2.0 * cave.RunSpeed * elapsedTime;
            }

            super.Update(elapsedTime);
        }

        public override function IsAlive():Boolean
        {
            if (!Dying)
            {
                return super.IsAlive();
            }
            else
            {
                return alpha > 0;
            }
        }
    }

}
