﻿package gamespecific
{
    import flash.display.Bitmap;
    import meyermike.Util;
    import flash.display.PixelSnapping;
    import meyermike.util.MathUtil;

    /**
     * Stalactites and stalagmites you must avoid
     * @author Mike Meyer
     */
    public class Obstacle extends CaveObject
    {
        [Embed(source='assets/stalactite_a.png')]
        private static const ImageA:Class;
        [Embed(source='assets/stalactite_b.png')]
        private static const ImageB:Class;
        [Embed(source='assets/stalactite_c.png')]
        private static const ImageC:Class;
        [Embed(source='assets/stalactite_d.png')]
        private static const ImageD:Class;

        private static const POSSIBLE_IMAGES:Array = [ImageA, ImageB, ImageC, ImageD];

        private var mGraphic:Bitmap;

        public override function Init():void
        {
            super.Init();
            CreateGraphics();

            mNearRed    = 0.85;
            mFarRed     = 0.40;
        }

        public override function Shutdown():void
        {
            super.Shutdown();
            DestroyGraphics();
        }

        protected function CreateGraphics():void
        {
            if (null == mGraphic)
            {
                var imageClass:Class = Util.RandomOneOf(POSSIBLE_IMAGES);
                mGraphic = new imageClass();
                mGraphic.smoothing = false;
                mGraphic.pixelSnapping = PixelSnapping.ALWAYS;
                mGraphic.cacheAsBitmap = false;

                this.addChild(mGraphic);
            }

            mGraphic.scaleX = (Math.random() * Math.random() * 2.5) + 0.5;
            mGraphic.scaleY = (Math.random() * Math.random() * 2.5) + 0.5;

            var theta:Number;

            if (Util.RandomBoolean())
            {
                // stalactite
                theta = Math.PI * MathUtil.Lerp( -0.15, -0.85, Math.random());
                mGraphic.x = (Global.TunnelRadius * Math.cos(theta));
                mGraphic.y = (Global.TunnelRadius * Math.sin(theta));
            }
            else
            {
                // stalagmite
                theta = Math.PI * MathUtil.Lerp( 0.15, 0.85, Math.random());
                mGraphic.x = (Global.TunnelRadius * Math.cos(theta));
                mGraphic.y = (Global.TunnelRadius * Math.sin(theta));
                mGraphic.scaleY = -1.0;
            }

            // center it
            mGraphic.x -= 0.5 * mGraphic.width;

            mXFlipped = false;
            mYFlipped = false;
        }

        protected function DestroyGraphics():void
        {
            this.removeChild(mGraphic);
            mGraphic = null;
        }

    }

}
