﻿package gamespecific
{
    import flash.display.Bitmap;
    import flash.display.DisplayObject;
    import flash.display.Sprite;
    import flash.geom.Point;
    import flash.ui.Keyboard;
    import meyermike.constants.Colors;
    import meyermike.InputUtil;
    import meyermike.PlatformerCharacter;
    import meyermike.util.ArrayUtil;
    import meyermike.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Player extends PlatformerCharacter
    {
        private const NUM_AIR_JUMPS:Number = 1;
        private const PLAYER_JUMP_SPEED:Number = 1250;
        private const PLAYER_JUMP_STOP_ACCEL:Number = PLAYER_JUMP_SPEED * 5.0;

        private var mAirJumpsRemaining:Number = NUM_AIR_JUMPS;

        private var mIsJumping:Boolean = false;
        public function get IsJumping():Boolean { return mIsJumping; }
        public function set IsJumping(value:Boolean):void { mIsJumping = value; }

        private var mTheta:Number = 0.5 * Math.PI;
        private static const STRAFE_SPEED:Number = 12.0;  // radians per second

        private var _allowUserControl:Boolean = true;
        public function get AllowUserControl():Boolean { return _allowUserControl; }
        public function set AllowUserControl(value:Boolean):void { _allowUserControl = value; }

        private var _cave:Cave;
        public function get cave():Cave { return _cave; }
        public function set cave(value:Cave):void { _cave = value; }

        // animation junk
        [Embed(source = 'assets/run_a.png')]
        private static const RunAClass:Class;
        [Embed(source = 'assets/run_b.png')]
        private static const RunBClass:Class;
        [Embed(source = 'assets/swing_a.png')]
        private static const SwingAClass:Class;
        [Embed(source = 'assets/swing_b.png')]
        private static const SwingBClass:Class;
        [Embed(source = 'assets/swing_c.png')]
        private static const SwingCClass:Class;

        private static const GraphicsClasses:Array = [RunAClass, RunBClass, SwingAClass, SwingBClass, SwingCClass];
        private static const RUN_A:Number = 0;
        private static const RUN_B:Number = 1;
        private static const SWING_A:Number = 2;
        private static const SWING_B:Number = 3;
        private static const SWING_C:Number = 4;
        private var mFrameGraphics:Array;

        private static const DISTANCE_PER_ANIM_FRAME:Number = 20;
        private var mAnimationFrame:Number = 0;
        private var mDistanceToNextFrame:Number = 0;

        public override function Init():void
        {
            super.Init();

            /// values for of platformer character acceleration
            mWalkAccel    = 3000;
            mAirAccel     = 3000;
            mStopAccel    = 3000;
            mMaxWalkSpeed =  600;

            CreateGraphics();
            Restart();

            // Ninjas don't even think twice about breaking the laws of physics
            PhysicsEnabled = false;
        }

        /**
         * per-frame update
         * @param   elapsedTime - time since the last frame
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            if (AllowUserControl)
            {
                Walk(elapsedTime);
                Jumping(elapsedTime);
                Animate(elapsedTime);
            }
        }

        /**
         * change dx based on user input
         */
        protected function Walk(elapsedTime:Number):void
        {
            var whereMouseIsAt:Point = InputUtil.GetMousePosition();
            var offsetToMouse:Point = whereMouseIsAt.subtract(Global.ScreenCenter);
            var targetTheta:Number = Math.atan2(offsetToMouse.y, offsetToMouse.x);

            var move:Number = targetTheta - mTheta;
            move = MathUtil.Wrap(move, -Math.PI, Math.PI);
            move = MathUtil.Clamp(move, -STRAFE_SPEED * elapsedTime, STRAFE_SPEED * elapsedTime);

            mTheta = MathUtil.Wrap(mTheta + move, -Math.PI, Math.PI);

            this.x = Global.ScreenCenter.x + (Global.TunnelRadius * Math.cos(mTheta));
            this.y = Global.ScreenCenter.y + (Global.TunnelRadius * Math.sin(mTheta));
            this.rotation = -90.0 + (mTheta * 180.0 / Math.PI);
        }

        /**
         *
         */
        protected function Jumping(elapsedTime:Number):void
        {
            /* DISABLING JUMPING FOR NOW
            if ( InputUtil.MouseButtonUp() || IsOnGround)
            {
                this.IsJumping = false;
            }

            // if we aren't holding the jump button anymore, cut
            // off the jump height
            if (!IsOnGround && !this.IsJumping && (this.dy < 0) )
            {
                this.dy += PLAYER_JUMP_STOP_ACCEL * elapsedTime;

                // don't shoot us down, though
                if (this.dy > 0)
                {
                    this.dy = 0;
                }
            }

            // starting a new jump
            if (InputUtil.MouseButtonJustPressed())
            {
                if (IsOnGround || (mAirJumpsRemaining > 0) )
                {
                    this.dy = -PLAYER_JUMP_SPEED;
                    this.IsJumping = true;
                }
                if (!IsOnGround)
                {
                    --mAirJumpsRemaining;
                }
            }

            // reset air jumps after landing
            if (IsOnGround)
            {
                mAirJumpsRemaining = NUM_AIR_JUMPS;
            }
            */
        }

        protected function CreateGraphics():void
        {
            if (null == mFrameGraphics)
            {
                mFrameGraphics = [];

                for each (var graphicClass:Class in GraphicsClasses)
                {
                    var frame:Bitmap = new graphicClass();
                    frame.smoothing = false;
                    frame.scaleX = 0.75;
                    frame.scaleY = 0.75;
                    frame.x = -0.5 * frame.width;
                    frame.y = -1.0 * frame.height;
                    this.addChild(frame);
                    mFrameGraphics.push(frame);
                }
            }

            mAnimationFrame = RUN_A;
            SwitchToAnimationFrame(mAnimationFrame);
        }

        public function SwingSword():void
        {
            mAnimationFrame = SWING_A;
        }

        /**
         * show the right animation frame
         */
        private function Animate(elapsedTime:Number):void
        {
            mDistanceToNextFrame -= cave.RunSpeed * elapsedTime;

            if (mDistanceToNextFrame <= 0)
            {
                mDistanceToNextFrame = DISTANCE_PER_ANIM_FRAME;

                // go to the next fram, whatever that is
                switch (mAnimationFrame)
                {
                    case RUN_A:
                        mAnimationFrame = RUN_B;
                        break;
                    case SWING_A:
                        mAnimationFrame = SWING_B;
                        break;
                    case SWING_B:
                        mAnimationFrame = SWING_C;
                        break;
                    default:
                        mAnimationFrame = RUN_A;
                        break;
                }
            }

            SwitchToAnimationFrame(mAnimationFrame);
        }

        /**
         * show/hide frames to match the specified enum value
         */
        private function SwitchToAnimationFrame(animationFrame:Number):void
        {
            for (var i:Number = 0; i < mFrameGraphics.length; ++i )
            {
                var frameGraphic:DisplayObject = mFrameGraphics[i];
                frameGraphic.visible = (i == animationFrame);
            }
        }
    }

}
