﻿package gamespecific.scripts
{
    import experimental.MochiLeaderboard;
    import flash.display.DisplayObject;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import gamespecific.Cave;
    import gamespecific.CaveObject;
    import gamespecific.Enemy;
    import gamespecific.FpsCounter;
    import gamespecific.Obstacle;
    import gamespecific.Plane;
    import gamespecific.Player;
    import gamespecific.widget.DisappearingText;
    import gamespecific.widget.Fader;
    import meyermike.constants.Colors;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.Global;
    import meyermike.script.Script;
    import meyermike.script.Step;
    import meyermike.script.step.Animate;
    import meyermike.script.step.Condition;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.Wait;
    import meyermike.Sounds;
    import meyermike.Util;
    import meyermike.util.MathUtil;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Game extends Script
    {
        [Embed(source='../assets/excitebikelike.mp3')]
        private static const IntroSoundClass:Class;
        private static var IntroSound:Sound = new IntroSoundClass();

        [Embed(source='../assets/go.mp3')]
        private static const GoSoundClass:Class;
        private static var GoSound:Sound = new GoSoundClass();

        [Embed(source='../assets/die.mp3')]
        private static const DieSoundClass:Class;
        private static var DieSound:Sound = new DieSoundClass();

        [Embed(source='../assets/strike.mp3')]
        private static const StrikeSoundClass:Class;
        private static var StrikeSound:Sound = new StrikeSoundClass();

        private static var SoundEffectsChannel:SoundChannel;

        private var mCave:Cave;
        private var mPlayer:Player;
        private var mFader:Fader;
        private var mLeaderBoard:MochiLeaderboard;

        private static const INITIAL_OBSTACLE_LEVEL:Number = 0.12;
        private static const OBSTACLE_LEVEL_STEP:Number = 0.005;
        private static const INTRO_RUN_SPEED_STEP:Number = 10;
        private static const RUN_SPEED_AT_GO:Number = 75;

        // extra fudge on collision detection.
        private static const Z_COLLISION_THICKNESS_ROCK:Number = 0;
        private static const Z_COLLISION_THICKNESS_ENEMY:Number = 20;

        private static const STRIKE_MESSAGES:Array = [
            "Got him!",
            "Got him!",
            "Got him!",
            "Got him!",
            "Got him!",
            "Got him!",
            "Got him!",
            "Nice!",
            "Killed!",
            "Sweet!",
            "Terminated!",
            "Enemy slain!",
            "Yes!",
            "murders++",
            "Murder Death Kill!",
            "Ninja'd!",
            "Bam!"];
        private static const ROCK_MESSAGES:Array = [
            "Don't run into rocks!",
            "Don't run into rocks!",
            "Watch out for rocks!",
            "Watch out for rocks!",
            "Avoid rocks, silly!",
            "Avoid rocks, silly!",
            "Rock'd!",
            "Less rock! More kill!",
            "Oops!"];

        private var mRunSpeedStep:Number = 11;
        private var mRunSpeedStepStep:Number = 5;

        private var mTutorialScript:Step;
        private var mMainGameScript:Step;
        private var mWinScript:Step;
        private var mLoseScript:Step;
        private var mCheckCollision:Boolean = true;

        /**
         * initial setup
         */
        public override function Begin(game:GameInterface):void
        {
            super.Begin(game);

            CreateScene();

            Game.Time.BeatsPerMinute = 60;

            if (null == mTutorialScript)
            {
                mTutorialScript = new Loop(1, [
                    new RunFunc(ShowControls),
                    new Wait(1),
                    new RunFunc(ShowGoal),
                    new Wait(1),
                    new RunFunc(ShowReady),
                    new RunFunc(ShowGo) ] );
                var mainGameSection:Step = new Loop(2, [ new Wait(1), new RunFunc(GetHarder) ]);
                mMainGameScript = new Loop(1, [
                    mainGameSection,
                    ProgressUpdate("Progress:  10%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  20%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  30%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  40%"),
                    mainGameSection,
                    ProgressUpdate("HALFWAY THERE!!!"),
                    mainGameSection,
                    ProgressUpdate("Progress:  60%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  70%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  80%"),
                    mainGameSection,
                    ProgressUpdate("Progress:  90%"),
                    mainGameSection,
                    ProgressUpdate("There's the end!"),
                    new RunFunc(EndCave),
                    new Condition(FadeComplete),
                    new RunFunc(YouWon) ] );
                mWinScript = new Loop(1, [
                    new RunFunc(ShowScore),
                    new RunFunc(ShowLeaderboard),
                    new Condition(DoneWithLeaderboard),
                    new RunFunc(ResetCave),
                    new RunFunc(ShowReady),
                    new RunFunc(ShowGo) ] ) ;
                mLoseScript = new Loop(1, [
                    // Lose() will have already been hit
                    new Condition(FadeComplete),
                    new RunFunc(ResetCave),
                    new RunFunc(ShowReady),
                    new RunFunc(ShowGo) ] );
            }

            mTutorialScript.Restart();
            mScript = mTutorialScript;
        }

        /**
         * create scene
         */
        protected function CreateScene():void
        {
            var w:Number = Global.StageWidth;
            var h:Number = Global.StageHeight;

            mFader = Game.Layers.UiLayer.Add(Fader) as Fader;

            mCave = new Cave();

            // player
            mPlayer = Game.Layers.PlayerLayer.Add(Player) as Player;
            mPlayer.cave = mCave;

            /*var fpsCounter:GameObject = this.Game.Layers.UiLayer.Add(FpsCounter);
            fpsCounter.x = 50;
            fpsCounter.y = 50;*/

            ResetCave();
        }

        /**
         * clean up
         */
        public override function End():void
        {
            super.End();
            Game.Layers.RemoveAll();
            mCave.Shutdown();
        }

        /**
         *
         */
        public override function Update():void
        {
            super.Update();
            mCave.Update(this.Game.Time.TimeDelta);
            CheckCollisions();
        }

        /**
         * detect and deal with it when the player runs into something
         */
        private function CheckCollisions():void
        {
            if (mCheckCollision)
            {
                this.Game.Layers.ForEachCartesianProductStyle(this.Game.Layers.PlayerLayer, this.Game.Layers.NpcLayer, CharacterNpcCollision);
            }
        }


        /**
         * detect and deal with it when the player runs into something
         */
        private function CharacterNpcCollision(player:Player, obstacle:CaveObject):void
        {
            if (obstacle is Enemy)
            {
                PlayerEnemyCollision(player, obstacle as Enemy);
            }
            else
            {
                PlayerRockCollision(player, obstacle);
            }
        }

        /**
         *
         */
        private function PlayerEnemyCollision(player:Player, enemy:Enemy):void
        {
            var front:Number = enemy.z - Z_COLLISION_THICKNESS_ENEMY;
            var back:Number =  enemy.z + (mCave.RunSpeed * this.Game.Time.TimeDelta) + Z_COLLISION_THICKNESS_ENEMY;

            if (!enemy.Dying)
            {
                if ( (front <= 0.0) && (0.0 <= back) )
                {
                    if (player.hitTestObject(enemy))
                    {
                        enemy.Dying = true;
                        StrikeEnemy(player, enemy as Enemy);
                        player.SwingSword();
                    }
                }
            }
        }

        /**
         *
         */
        private function PlayerRockCollision(player:Player, obstacle:CaveObject):void
        {
            var obstacleFront:Number = obstacle.z - Z_COLLISION_THICKNESS_ROCK;
            var obstacleBack:Number =  obstacle.z + (mCave.RunSpeed * this.Game.Time.TimeDelta) + Z_COLLISION_THICKNESS_ROCK;

            if ( (obstacleFront <= 0.0) && (0.0 <= obstacleBack) )
            {
                // HACKSVILLE - large chunks of the image are empty, but colliding anyway.
                player.scaleX *= 0.25;
                player.scaleY *= 0.25;

                if (player.hitTestObject(obstacle))
                {
                    Lose(Util.RandomOneOf(ROCK_MESSAGES));
                }

                // HACKSTOWNE - undo above hack
                player.scaleX *= 4.0;
                player.scaleY *= 4.0;
            }
        }

        /**
         *
         */
        private function ShowControls():void
        {
            AddDisappearingTextField("Move the mouse", 0.5, 0.45);
            AddDisappearingTextField("to control", 0.5, 0.55);

            SoundEffectsChannel = Sounds.PlaySound(IntroSound, SoundEffectsChannel, 0.5);
            mCave.RunSpeed += INTRO_RUN_SPEED_STEP;
            mCave.ObstacleLevel = INITIAL_OBSTACLE_LEVEL;
        }

        /**
         * tell them whant the goal is
         */
        private function ShowGoal():void
        {
            AddDisappearingTextField("Hit as many enemies", 0.5, 0.45);
            AddDisappearingTextField("as you can!", 0.5, 0.55);

            SoundEffectsChannel = Sounds.PlaySound(IntroSound, SoundEffectsChannel, 0.5);
            mCave.RunSpeed += INTRO_RUN_SPEED_STEP;
            mCave.ObstacleLevel = INITIAL_OBSTACLE_LEVEL;
        }

        /**
         *
         */
        private function ShowReady():void
        {
            AddDisappearingTextField("Ready?", 0.5, 0.5, 1.0, 0.25);

            SoundEffectsChannel = Sounds.PlaySound(IntroSound, SoundEffectsChannel, 0.5);
            mCave.RunSpeed += INTRO_RUN_SPEED_STEP;
        }

        /**
         *
         */
        private function ShowGo():void
        {
            AddDisappearingTextField("GO!", 0.5, 0.5);

            SoundEffectsChannel = Sounds.PlaySound(GoSound, SoundEffectsChannel, 0.5);
            mCave.RunSpeed = RUN_SPEED_AT_GO;
            mCave.ObstacleLevel = INITIAL_OBSTACLE_LEVEL;

            mMainGameScript.Restart();
            mScript = mMainGameScript;
        }

        /**
         *
         */
        private function ShowHereWeGoAgain():void
        {
            AddDisappearingTextField("Here we go again!", 0.5, 0.45, 2.0, 1.0, Colors.BLACK_ENOUGH);
            AddDisappearingTextField("Faster this time!", 0.5, 0.55, 2.0, 1.0, Colors.BLACK_ENOUGH);

            mRunSpeedStep += mRunSpeedStepStep;
        }


        /**
         *
         */
        private function GetHarder():void
        {
            mCave.ObstacleLevel += OBSTACLE_LEVEL_STEP;
            mCave.RunSpeed += mRunSpeedStep;
        }

        /**
         *
         */
        private function EndCave():void
        {
            mCave.CaveBuildingEnabled = false;
            mFader.FadeToColor(this.Game.Time.SecondsPerBeat * 0.9, Colors.WHITE_ENOUGH);
        }

        /**
         *
         */
        private function YouWon():void
        {
            mWinScript.Restart();
            mScript = mWinScript;

            AddDisappearingTextField("You made it!", 0.5, 0.5, 1.0, 0.5, Colors.BLACK_ENOUGH);
        }

        /**
         *
         */
        private function ShowScore():void
        {
            if ( (mCave.NumFoesSpawned > 0) && (mCave.NumFoesDefeated==mCave.NumFoesSpawned) )
            {
                AddDisappearingTextField("HOLY CRAP",            0.5, 0.3, 2.0, 1.0, Colors.BLACK_ENOUGH);
                AddDisappearingTextField("100%!?",               0.5, 0.4, 2.0, 1.0, Colors.BLACK_ENOUGH);
                AddDisappearingTextField("I can't do that!",     0.5, 0.6, 2.0, 1.0, Colors.BLACK_ENOUGH);
                AddDisappearingTextField("And I made the game!", 0.5, 0.7, 2.0, 1.0, Colors.BLACK_ENOUGH);
            }
            else
            {
                var percent:Number = 0;
                if (mCave.NumFoesSpawned > 0)
                {
                    percent = 100.0 * mCave.NumFoesDefeated / mCave.NumFoesSpawned;
                }
                var ratioStr:String = String(mCave.NumFoesDefeated) + " out of " + String(mCave.NumFoesSpawned);
                var percentStr:String = String(Math.floor(percent))+"%";

                AddDisappearingTextField("You defeated", 0.5, 0.4, 2.0, 1.0, Colors.BLACK_ENOUGH);
                AddDisappearingTextField(ratioStr, 0.5, 0.5, 2.0, 1.0, Colors.BLACK_ENOUGH);
                AddDisappearingTextField(percentStr, 0.5, 0.6, 2.0, 1.0, Colors.BLACK_ENOUGH);
            }
        }

        /**
         *
         */
        private function Lose(loseMessage:String):void
        {
            AddDisappearingTextField(loseMessage, 0.5, 0.5);

            mCave.RunSpeed = 0;

            mFader.FadeToColor(this.Game.Time.SecondsPerBeat * 0.9, Colors.BLACK_ENOUGH);

            mLoseScript.Restart();
            mScript = mLoseScript;

            mCheckCollision = false;
            mPlayer.AllowUserControl = false;

            SoundEffectsChannel = Sounds.PlaySound(DieSound, SoundEffectsChannel, 0.5);
        }

        /**
         *
         */
        private function StrikeEnemy(player:Player, enemy:Enemy):void
        {
            AddDisappearingTextField(
                Util.RandomOneOf(STRIKE_MESSAGES),
                MathUtil.Lerp(0.45, 0.55, Math.random()),
                MathUtil.Lerp(0.3, 0.7, Math.random()) );
            SoundEffectsChannel = Sounds.PlaySound(StrikeSound, SoundEffectsChannel, 0.5);
        }

        /**
         * add a disappearing text field to the scene
         */
        private function AddDisappearingTextField(text:String, x_pct:Number = 0.5, y_pct:Number = 0.5, time:Number = 2.0, fadeTime:Number = 1.0, color:uint = Colors.WHITE_ENOUGH):void
        {
            var w:Number = meyermike.Global.StageWidth;
            var h:Number = meyermike.Global.StageHeight;

            var disappearingText:DisappearingText = this.Game.Layers.UiLayer.Add(DisappearingText) as DisappearingText;
            disappearingText.TimeRemaining = time;
            disappearingText.FadeTime = fadeTime;
            disappearingText.SetFontSize(24);
            disappearingText.SetFontColor(color);
            disappearingText.SetText(text);
            disappearingText.CenterOn( x_pct * w, y_pct * h );
        }

        /**
         * used with Condition steps to wait until the fade is over
         */
        private function FadeComplete():Boolean
        {
            return !mFader.FadeInProgress;
        }

        /**
         * reset cave for a new game
         */
        private function ResetCave():void
        {
            this.Game.Layers.PlatformLayer.RemoveAll();
            this.Game.Layers.NpcLayer.RemoveAll();

            mCave.RunSpeed = 0;
            mCave.ObstacleLevel = 0;
            mCave.Shutdown();
            mCave.Init(this.Game.Layers.PlatformLayer, this.Game.Layers.NpcLayer);

            mFader.FadeBack(this.Game.Time.SecondsPerBeat * 0.9);

            mCheckCollision = true;
            mPlayer.AllowUserControl = true;
        }

        /**
         * make a script step for putting a progress update on screen
         */
        private function ProgressUpdate(text:String):Step
        {
            return new RunFunc(function():void {
                AddDisappearingTextField(text, 0.5, 0.5, 2.0, 2.0, Colors.DESAT_YELLOW);
                SoundEffectsChannel = Sounds.PlaySound(IntroSound, SoundEffectsChannel, 0.5); } );
        }

        /**
         *
         */
        private function ShowLeaderboard():void
        {
            trace("?");
            if (null == mLeaderBoard)
            {
                mLeaderBoard = this.Game.Layers.UiLayer.Add(MochiLeaderboard) as MochiLeaderboard;
            }

            var percent:Number = 0;
            if (mCave.NumFoesSpawned > 0)
            {
                percent = 100.0 * mCave.NumFoesDefeated / mCave.NumFoesSpawned;
            }

            // push leaderboard to the top
            mLeaderBoard.parent.setChildIndex( mLeaderBoard, mLeaderBoard.parent.numChildren-1 );

            mLeaderBoard.SubmitScoreAndShowLeaderboard(percent);
        }

        /**
         *
         */
        private function DoneWithLeaderboard():Boolean
        {
            if (null == mLeaderBoard)
            {
                ShowHereWeGoAgain();
                return true;
            }
            else
            {
                if (!mLeaderBoard.LeaderboardUp)
                {
                    ShowHereWeGoAgain();
                    return true;
                }

                return false;
            }
        }
    }

}
