﻿package meyermike
{
    import flash.display.Stage;
    import flash.events.KeyboardEvent;
    import flash.events.MouseEvent;
    import flash.geom.Point;
    import flash.ui.Keyboard;

    /**
     * ...
     * @author Mike Meyer
     */
    public class InputUtil
    {
        public static const KEYSTATE_UP:uint           = 0;
        public static const KEYSTATE_JUSTPRESSED:uint  = 1;
        public static const KEYSTATE_HELD:uint         = 2;
        public static const KEYSTATE_JUSTRELEASED:uint = 3;

        private static var sKeyStates:Object = new Object();
        private static var sMouseButtonState:Number = InputUtil.KEYSTATE_UP;
        private static var sMousePosition:Point         = new Point();
        private static var sPreviousMousePosition:Point = new Point();
        private static var sStage:Stage;

        /**
         * Get InputUtil ready to be used
         */
        public static function Init(stage:Stage):void
        {
            sStage = stage;
            sStage.addEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);
            sStage.addEventListener(KeyboardEvent.KEY_UP, OnKeyUp);
            sStage.addEventListener(MouseEvent.MOUSE_DOWN, MouseDown);
            sStage.addEventListener(MouseEvent.MOUSE_MOVE, MouseMove);
            sStage.addEventListener(MouseEvent.MOUSE_UP, MouseUp);
            sStage.addEventListener(MouseEvent.ROLL_OUT, MouseUp);
        }

        /**
         * clean up
         */
        public static function Shutdown():void
        {
            sStage.removeEventListener(KeyboardEvent.KEY_DOWN, OnKeyDown);
            sStage.removeEventListener(KeyboardEvent.KEY_UP, OnKeyUp);
            sStage.removeEventListener(MouseEvent.MOUSE_DOWN, MouseDown);
            sStage.removeEventListener(MouseEvent.MOUSE_MOVE, MouseMove);
            sStage.removeEventListener(MouseEvent.MOUSE_UP, MouseUp);
            sStage.removeEventListener(MouseEvent.ROLL_OUT, MouseUp);
        }

        /**
         * make the JUST X'ed states go to the more permanent ones
         */
        public static function Update():void
        {
            for (var key:String in InputUtil.sKeyStates)
            {
                switch (InputUtil.sKeyStates[key])
                {
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                        InputUtil.sKeyStates[key] = InputUtil.KEYSTATE_HELD;
                        break;
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        InputUtil.sKeyStates[key] = InputUtil.KEYSTATE_UP;
                        break;
                }
            }

            // mouse button state
            switch (sMouseButtonState)
            {
                case InputUtil.KEYSTATE_JUSTPRESSED:
                    sMouseButtonState = InputUtil.KEYSTATE_HELD;
                    break;
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    sMouseButtonState = InputUtil.KEYSTATE_UP;
                    break;
            }

            // mouse position
            sPreviousMousePosition.x = sMousePosition.x;
            sPreviousMousePosition.y = sMousePosition.y;
            sMousePosition.x = sStage.mouseX;
            sMousePosition.y = sStage.mouseY;
        }

        /**
         * update the states when a key is pressed
         */
        private static function OnKeyDown(e:KeyboardEvent):void
        {
            if (InputUtil.sKeyStates.hasOwnProperty(e.keyCode))
            {
                switch (InputUtil.sKeyStates[e.keyCode])
                {
                    case InputUtil.KEYSTATE_UP:
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        InputUtil.sKeyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTPRESSED;
                        break;
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                    case InputUtil.KEYSTATE_HELD:
                        // stay there, let Update handle it
                        break;
                }
            }
            else
            {
                InputUtil.sKeyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTPRESSED;
            }
        }

        /**
         * update the states when a key is released
         */
        private static function OnKeyUp(e:KeyboardEvent):void
        {
            if (InputUtil.sKeyStates.hasOwnProperty(e.keyCode))
            {
                switch (InputUtil.sKeyStates[e.keyCode])
                {
                    case InputUtil.KEYSTATE_UP:
                    case InputUtil.KEYSTATE_JUSTRELEASED:
                        // stay there
                        break;
                    case InputUtil.KEYSTATE_JUSTPRESSED:
                    case InputUtil.KEYSTATE_HELD:
                        InputUtil.sKeyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTRELEASED;
                        break;
                }
            }
            else
            {
                InputUtil.sKeyStates[e.keyCode] = InputUtil.KEYSTATE_JUSTRELEASED;
            }
        }

        /**
         * tell whether any of the keys in the array are down
         */
        public static function AnyDown(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (IsDown(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array are up
         */
        public static function AnyUp(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (IsUp(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array have been up for more than a frame
         */
        public static function AnyReleased(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (IsReleased(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array have been down for more than a frame
         */
        public static function AnyHeld(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (IsHeld(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array were just pressed
         */
        public static function AnyJustPressed(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (WasJustPressed(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether any of the keys in the array were just released
         */
        public static function AnyJustReleased(keyCodes:/*uint*/Array):Boolean
        {
            for each (var keyCode:uint in keyCodes)
            {
                if (WasJustReleased(keyCode))
                {
                    return true;
                }
            }
            return false;
        }

        /**
         * tell whether a key is down or not
         */
        public static function IsDown(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return ( (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_HELD) || (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_JUSTPRESSED) );
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key is up or not
         */
        public static function IsUp(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return ( (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_UP) || (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_JUSTRELEASED) );
            }
            else
            {
                return true;
            }
        }

        /**
         * tell whether a key has been released for a frame or more
         */
        public static function IsReleased(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_UP);
            }
            else
            {
                return true;
            }
        }

        /**
         * tell whether a key has been held for a frame or more
         */
        public static function IsHeld(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_HELD);
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key was just pressed this frame
         */
        public static function WasJustPressed(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_JUSTPRESSED);
            }
            else
            {
                return false;
            }
        }

        /**
         * tell whether a key was just released this frame
         */
        public static function WasJustReleased(keyCode:uint):Boolean
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return (InputUtil.sKeyStates[keyCode] == InputUtil.KEYSTATE_JUSTRELEASED);
            }
            else
            {
                return false;
            }
        }

        /**
         * get the keystate directly
         */
        public static function GetKeyState(keyCode:uint):uint
        {
            if (InputUtil.sKeyStates.hasOwnProperty(keyCode))
            {
                return InputUtil.sKeyStates[keyCode];
            }
            else
            {
                return InputUtil.KEYSTATE_UP;
            }
        }

        /**
         * deal with mouse down events -- start mouse-looking
         */
        private static function MouseDown(e:MouseEvent):void
        {
            switch (sMouseButtonState)
            {
                case InputUtil.KEYSTATE_UP:
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    sMouseButtonState = InputUtil.KEYSTATE_JUSTPRESSED;
                    break;
                case InputUtil.KEYSTATE_JUSTPRESSED:
                case InputUtil.KEYSTATE_HELD:
                    // stay there, let Update handle it
                    break;
            }
        }

        /**
         * deal with mouse down events -- stop mouse-looking
         */
        private static function MouseUp(e:MouseEvent):void
        {
            switch (sMouseButtonState)
            {
                case InputUtil.KEYSTATE_UP:
                case InputUtil.KEYSTATE_JUSTRELEASED:
                    // stay there
                    break;
                case InputUtil.KEYSTATE_JUSTPRESSED:
                case InputUtil.KEYSTATE_HELD:
                    sMouseButtonState = InputUtil.KEYSTATE_JUSTRELEASED;
                    break;
            }
        }

        /**
         * deal with mouse down events -- mouse-looking
         */
        private static function MouseMove(e:MouseEvent):void
        {
        }

        /**
         * tell whether the mouse button is down or not
         */
        public static function MouseButtonDown():Boolean
        {
            return ( (sMouseButtonState == InputUtil.KEYSTATE_HELD) || (sMouseButtonState== InputUtil.KEYSTATE_JUSTPRESSED) );
        }

        /**
         * tell whether the mouse button is up or not
         */
        public static function MouseButtonUp():Boolean
        {
            return ( (sMouseButtonState == InputUtil.KEYSTATE_UP) || (sMouseButtonState== InputUtil.KEYSTATE_JUSTRELEASED) );
        }

        /**
         * tell whether the mouse button has been held more than a frame
         */
        public static function MouseButtonHeld():Boolean
        {
            return (sMouseButtonState == InputUtil.KEYSTATE_HELD);
        }

        /**
         * tell whether the mouse button button was just pressed this frame
         */
        public static function MouseButtonJustPressed():Boolean
        {
            return (sMouseButtonState == InputUtil.KEYSTATE_JUSTPRESSED);
        }

        /**
         * get the mouse position
         */
        public static function GetMousePosition():Point
        {
            return sMousePosition.clone();
        }

        /**
         * get how much the mouse moved this frame
         */
        public static function GetMouseMovement():Point
        {
            return sMousePosition.subtract(sPreviousMousePosition);
        }
    }
}
