﻿package meyermike
{
    import flash.display.DisplayObject;
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.constants.Defaults;
    import meyermike.core.GameInterface;
    import meyermike.core.GameObject;
    import meyermike.util.MathUtil;
    import meyermike.widget.TextField;

    /**
     * ...
     * @author Mike Meyer
     */
    public class Util
    {
        // the farthest (relative) distance we want to go between collision tests
        private static const COLLISION_STEP_SIZE_MAX:Number = 1;

        /**
         * return a random one of the things in an array
         */
        public static function RandomOneOf(a:Array):*
        {
            return a[int(Math.random() * a.length)];
        }

        /**
         * randomly true or false
         */
        public static function RandomBoolean():Boolean
        {
            return (Math.random() < 0.5);
        }

        /**
         * check for collisions between two moving GameObjects
         */
        public static function CheckAabbCollision(objectA:GameObject, objectB:GameObject, elapsedTime:Number):CollisionInfo
        {
            var objectABounds:Rectangle = objectA.GetCollisionBounds();
            var objectBBounds:Rectangle = objectB.GetCollisionBounds();

            return Util._InternalCheckAabbCollision(
                objectABounds, new Point(objectA.dx * elapsedTime, objectA.dy * elapsedTime),
                objectBBounds, new Point(objectB.dx * elapsedTime, objectB.dy * elapsedTime) );
        }

        /**
         * check for collisions between two potentially moving rects
         */
        private static function _InternalCheckAabbCollision(
            objectABounds:Rectangle,
            objectAMotion:Point,
            objectBBounds:Rectangle,
            objectBMotion:Point ):CollisionInfo
        {
            var dist:Number = objectAMotion.subtract(objectBMotion).length;
            var steps:Number = 1 + Math.floor(dist / COLLISION_STEP_SIZE_MAX);

            for (var step:Number = 0;  step <= steps; ++step)
            {
                var scale:Number = (step) / steps;
                var a:Rectangle = objectABounds.clone();
                a.offset(objectAMotion.x * scale, objectAMotion.y * scale);
                var b:Rectangle = objectBBounds.clone();
                b.offset(objectBMotion.x * scale, objectBMotion.y * scale);

                if (a.intersects(b))
                {
                    var ci:CollisionInfo =
                        _CreateCollisionInfoForAabbCollision(
                            a,
                            b,
                            objectAMotion.x - objectBMotion.x,
                            objectAMotion.y - objectBMotion.y);
                    ci.t = step / steps;
                    return ci;
                }
            }

            return null;
        }

        /**
         * used internally by CheckAabbCollision
         */
        private static function _CreateCollisionInfoForAabbCollision(
            objectA:Rectangle,
            objectB:Rectangle,
            xmove:Number, ymove:Number):CollisionInfo
        {
            var a_center:Point = MathUtil.GetRectCenter(objectA);
            var b_center:Point = MathUtil.GetRectCenter(objectB);
            var ci:CollisionInfo = new CollisionInfo();

            // defaults
            ci.location.x = a_center.x;
            ci.location.y = a_center.y;
            ci.normal.x = 0.0;
            ci.normal.y = 0.0;

            var xdiff:Number = (xmove > 0.0) ? (objectA.right - objectB.left) : (objectB.right - objectA.left);
            var ydiff:Number = (ymove > 0.0) ? (objectA.bottom - objectB.top) : (objectB.bottom - objectA.top);

            var x_first:Boolean = ( Math.abs(xmove) > Math.abs(ymove) );
            var checkNextAxis:Boolean = true;

            if (x_first && checkNextAxis)
            {
                if ( xmove > 0 )
                {
                    if ( objectB.containsPoint( new Point( objectA.right, a_center.y ) ) )
                    {
                        ci.location.x = objectB.left;
                        ci.normal.x = -1;
                        ci.normal.y =  0;
                        checkNextAxis = false;
                    }
                }
                else
                {
                    if ( objectB.containsPoint( new Point( objectA.left, a_center.y ) ) )
                    {
                        ci.location.x = objectB.right;
                        ci.normal.x =  1;
                        ci.normal.y =  0;
                        checkNextAxis = false;
                    }
                }
            }

            if (checkNextAxis)
            {
                if (ymove > 0)
                {
                    if ( objectB.containsPoint( new Point( a_center.x, objectA.bottom ) ) )
                    {
                        ci.location.y = objectB.top;
                        ci.normal.x =  0;
                        ci.normal.y = -1;
                        checkNextAxis = false;
                    }
                }
                else
                {
                    if ( objectB.containsPoint( new Point( a_center.x, objectA.top ) ) )
                    {
                        ci.location.y = objectB.bottom;
                        ci.normal.x =  0;
                        ci.normal.y =  1;
                        checkNextAxis = false;
                    }
                }
            }

            if (!x_first && checkNextAxis)
            {
                if ( xmove > 0 )
                {
                    if ( objectB.containsPoint( new Point( objectA.right, a_center.y ) ) )
                    {
                        ci.location.x = objectB.left;
                        ci.normal.x = -1;
                        ci.normal.y =  0;
                        checkNextAxis = false;
                    }
                }
                else
                {
                    if ( objectB.containsPoint( new Point( objectA.left, a_center.y ) ) )
                    {
                        ci.location.x = objectB.right;
                        ci.normal.x =  1;
                        ci.normal.y =  0;
                        checkNextAxis = false;
                    }
                }
            }

            // do yet more checking now that the "obvious" cases are handled.
            if (checkNextAxis)
            {
                // keep the very edges of things from causing unwanted collisions
                const FUDGE:Number = 4;

                var xoverlaps:Boolean = (objectA.right - FUDGE  > objectB.left) && (objectA.left + FUDGE < objectB.right);
                var yoverlaps:Boolean = (objectA.bottom - FUDGE > objectB.top)  && (objectA.top + FUDGE < objectB.bottom);

                if (x_first && checkNextAxis && yoverlaps)
                {
                    if (xmove > 0)
                    {
                        if ( (objectA.right > objectB.left) && (objectA.right < objectB.right)
                          && ((objectA.right - xmove) < objectB.left) ) // make sure it was this frame that it crossed the edge
                        {
                            ci.location.x = objectB.left;
                            ci.normal.x = -1;
                            ci.normal.y =  0;
                            checkNextAxis = false;
                        }
                    }
                    else if (xmove < 0)
                    {
                        if ( (objectA.left > objectB.left) && (objectB.left < objectB.right)
                          && ( (objectA.left - xmove) > objectB.right) ) // make sure it was this frame that it crossed the edge
                        {
                            ci.location.x = objectB.right;
                            ci.normal.x =  1;
                            ci.normal.y =  0;
                            checkNextAxis = false;
                        }
                    }
                }

                if (checkNextAxis && xoverlaps)
                {
                    if (ymove > 0)
                    {
                        if ( (objectA.bottom > objectB.top) && (objectA.bottom < objectB.bottom)
                          && ( (objectA.bottom - ymove) < objectB.top) ) // make sure it was this frame that it crossed the edge
                        {
                            ci.location.y = objectB.top;
                            ci.normal.x =  0;
                            ci.normal.y = -1;
                            checkNextAxis = false;
                        }
                    }
                    else if (ymove < 0)
                    {
                        if ( (objectA.top > objectB.top) && (objectB.top < objectB.bottom)
                          && ( (objectA.top - ymove) > objectB.bottom) ) // make sure it was this frame that it crossed the edge
                            {
                            ci.location.y = objectB.bottom;
                            ci.normal.x =  0;
                            ci.normal.y =  1;
                            checkNextAxis = false;
                        }
                    }
                }

                if (!x_first && checkNextAxis && yoverlaps)
                {
                    if (xmove > 0)
                    {
                        if ( (objectA.right > objectB.left) && (objectA.right < objectB.right)
                          && ((objectA.right - xmove) < objectB.left) ) // make sure it was this frame that it crossed the edge
                        {
                            ci.location.x = objectB.left;
                            ci.normal.x = -1;
                            ci.normal.y =  0;
                            checkNextAxis = false;
                        }
                    }
                    else if (xmove < 0)
                    {
                        if ( (objectA.left > objectB.left) && (objectB.left < objectB.right)
                          && ( (objectA.left - xmove) > objectB.right) ) // make sure it was this frame that it crossed the edge
                        {
                            ci.location.x = objectB.right;
                            ci.normal.x =  1;
                            ci.normal.y =  0;
                            checkNextAxis = false;
                        }
                    }
                }
            }

            ci.t = 0.0; // needs to be set outside this function

            return ci;
        }

        /**
         * take a displayobject and stretch it between two points
         * ASSUMES the object's registration point is in its center
         */
        public static function StretchBetween(object:DisplayObject, pointA:Point, pointB:Point, maxLen:Number = 0.0):void
        {
            object.rotation = 0;
            var distance:Number = Point.distance(pointA, pointB);
            if (distance == 0.0)
            {
                distance = 1.0;
            }

            var scale:Number = 1.0;

            if ( (0.0 != maxLen) && (distance > maxLen) )
            {
                scale = maxLen / distance;
            }

            object.width = distance * scale;

            var center:Point = Point.interpolate(pointB, pointA, 0.5 * scale);
            object.x = center.x;
            object.y = center.y;

            object.rotation = Math.atan2(pointB.y - pointA.y, pointB.x - pointA.x) * 180.0 / Math.PI;
        }


        /**
         * Add a textField to the stage
         */
        public static function AddTextfield(
            game:GameInterface,
            text:String,
            x_pct:Number,
            y_pct:Number,
            fontSize:Number,
            font:String=null ):meyermike.widget.TextField
        {
            // todo: font defaults stuff should be coming from some config class

            var textField:meyermike.widget.TextField = game.Layers.UiLayer.Add(meyermike.widget.TextField) as meyermike.widget.TextField;

            var stageWidth:Number = Global.StageWidth;
            var stageHeight:Number = Global.StageHeight;

            textField.SetText(text);
            textField.SetFontSize(fontSize);
            textField.SetFontColor(Defaults.FONT_COLOR);
            textField.x = x_pct * stageWidth;
            textField.y = y_pct * stageHeight;
            if (null != font)
            {
                textField.SetFont(font);
            }
            else
            {
                textField.SetFont(Defaults.FONT);
            }

            return textField;
        }
    }
}
