﻿package meyermike.collision
{
    import flash.display.DisplayObjectContainer;
    import flash.geom.Point;

    /**
     * Struct used to represent the important information about a collision
     * once one has been detected
     * @author Mike Meyer
     */
    public class CollisionInfo
    {
        /**
         * where the collision occurred
         */
        public var location:Point;

        /**
         * Normal (pointing toward the first object)
         */
        public var normal:Point;

        /**
         * percentage of the movement that it got through before
         * collision [0,1]
         */
        public var t:Number;

        public function CollisionInfo(location:Point=null, normal:Point=null, t:Number=0)
        {
            this.location = location;
            this.normal = normal; 
            this.t = t;
            
            if (null == this.normal)
            {
                this.normal = new Point();
            }
            if (null == this.location)
            {
                this.location = new Point();
            }
        }
        
        /**
         * Take a CollisionInfo made for the global coordinate system and 
         * return a new one in the coordinates of the given object
         */
        public function GlobalToLocal(object:DisplayObjectContainer):CollisionInfo
        {
            return new CollisionInfo( 
                object.globalToLocal(this.location),
                object.globalToLocal(this.normal).subtract(object.globalToLocal(new Point(0, 0))),
                this.t);
        }
    }

}