package meyermike.util
{
    import flash.events.Event;
    import flash.display.Stage;
    import flash.events.KeyboardEvent;
    import flash.utils.Dictionary;

    /**
     * [broadcast event] Dispatched when Flash Player loses operating system focus and is becoming inactive.
     * @eventType meyermike.util.Cheat.CHEATENTERED
     */
    [Event(name="cheatentered", type="flash.events.Event")]

    /**
     * Utility class for adding cheat codes.
     * Register the string to type, and associate a callback with it.
     *
     * Example use:
     *
     *      var cheats:CheatUtil = new CheatUtil();
     *      cheats.Init(stage);
     *      cheats.AddCheatCode("next", Music.StartNewMusic);
     *
     * @author Mike Meyer
     */
    public class CheatUtil
    {
        private var mStage:Stage;
        private var mKeysEntered:String = "";
        private static const KEYLOG_MAX_LENGTH:uint = 16;

        /// used as a hash from cheatCodes(strings) to callbacks(arrays of functions)
        private var mCheats:Dictionary;

        /**
         * Get the Cheat utility ready for use
         * @param	stage - stage instance.  used so that this thing can
         *                  listen to keyDown
         */
        public function Init(stage:Stage):void
        {
            mStage = stage;
            mStage.addEventListener(KeyboardEvent.KEY_DOWN, HandleKeyDown);

            mCheats = new Dictionary();
        }

        /**
         * Clean up the cheat util
         */
        public function Shutdown():void
        {
            mStage.removeEventListener(KeyboardEvent.KEY_DOWN, HandleKeyDown);
            mStage = null;

            mCheats = null;
        }

        /**
         * Add a cheat code to the util
         * @param	code      string to type to make your callback get called
         * @param	callback  callback to make the cheat happen with
         */
        public function AddCheatCode(code:String, callback:Function):void
        {
            if (0 < code.length)
            {
                if (mCheats.hasOwnProperty(code))
                {
                    var callbacks:Array = mCheats[code];
                    if (-1 == callbacks.indexOf(callback))
                    {
                        callbacks.push(callback);
                    }
                }
                else
                {
                    mCheats[code] = [callback];
                }
            }
        }

        /**
         * remove a cheat code from the util
         * @param	code      should match the AddCheatCode cal you are undoing
         * @param	callback  should match the AddCheatCode cal you are undoing
         */
        public function RemoveCheatCode(code:String, callback:Function):void
        {
            if (mCheats.hasOwnProperty(code))
            {
                var callbacks:Array = mCheats[code];
                var index:int = callbacks.indexOf(callback);
                if ( -1 != index )
                {
                    callbacks.splice(index, 1);
                }
                if (0 == callbacks.length)
                {
                    delete mCheats[code];
                }
            }
        }

        /**
         * watch for the key presses, and when a cheat code has been typed, fire off the appropriate callbacks
         */
        private function HandleKeyDown(e:KeyboardEvent):void
        {
            if (0 != e.charCode)
            {
                mKeysEntered += String.fromCharCode(e.charCode);

                if (mKeysEntered.length > KEYLOG_MAX_LENGTH)
                {
                    mKeysEntered = mKeysEntered.substring(1);
                }

                for (var cheat:String in mCheats)
                {
                    if ( StringUtil.EndsWith( mKeysEntered, cheat ) )
                    {
                        var callbacks:Array = mCheats[cheat];
                        for each (var callback:Function in callbacks)
                        {
                            callback();
                        }
                    }
                }
            }
        }
    }
}
