package meyermike.util
{
    import flash.events.MouseEvent;
    import flash.geom.Rectangle;
    import meyermike.constants.Defaults;
    import meyermike.core.GameObjectHolder;
    import meyermike.widget.PulsingTextButton;
    import meyermike.widget.TextField;

    /**
     * Utility for putting simple guis together easily
     * @author Mike Meyer
     */
    public class GuiBuilder
    {
        private var _font:String;
        public function get Font():String { return _font; }
        public function set Font(value:String):void { _font = value; }

        private var _fontColor:Number;
        public function get FontColor():Number { return _fontColor; }
        public function set FontColor(value:Number):void { _fontColor = value; }

        private var _highlightColor:Number;
        public function get HighlightColor():Number { return _highlightColor; }
        public function set HighlightColor(value:Number):void { _highlightColor = value; }

        private var _fontSize:Number;
        public function get FontSize():Number { return _fontSize; }
        public function set FontSize(value:Number):void { _fontSize = value; }

        private var _layer:GameObjectHolder;
        public function get Layer():GameObjectHolder { return _layer; }
        public function set Layer(value:GameObjectHolder):void { _layer = value; }

        private var mArea:Rectangle;

        function GuiBuilder(usableArea:Rectangle, layer:GameObjectHolder)
        {
            _font = Defaults.FONT;
            _fontColor = Defaults.FONT_COLOR;
            _fontSize = Defaults.FONT_SIZE;
            _highlightColor = Defaults.HIGHLIGHT_COLOR;
            _layer = layer;
            mArea = usableArea;
        }

        /**
         * helper for the add text functions
         * handles everything but the positioning
         */
        private function _AddText(text:String):TextField
        {
            var tf:TextField = _layer.Add(TextField) as TextField;
            tf.SetText(text);
            tf.SetFont(Font);
            tf.SetFontColor(FontColor);
            tf.SetFontSize(FontSize);
            return tf;
        }

        /**
         * helper for the add button functions
         * handles everything but the positioning
         */
        private function _AddButton(text:String, callback:Function):PulsingTextButton
        {
            var b:PulsingTextButton = _layer.Add(PulsingTextButton) as PulsingTextButton;
            b.SetText(text);
            b.SetFont(Font);
            b.SetFontColor(FontColor);
            b.SetFontSize(FontSize);
            b.SetHoverColor(HighlightColor);
            b.addEventListener(MouseEvent .CLICK, callback);
            return b;
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddText(text:String, x_pct:Number, y_pct:Number):void
        {
            var tf:TextField = _AddText(text);
            tf.x = mArea.left + (mArea.width * x_pct);
            tf.y = mArea.top + (mArea.height * y_pct);
        }

        /**
         * Add a textfield to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddTextCentered(text:String, x_pct:Number, y_pct:Number):void
        {
            var tf:TextField = _AddText(text);
            tf.CenterOn( mArea.left + (mArea.width * x_pct),
                         mArea.top + (mArea.height * y_pct) );
        }

        /**
         * Add a button to the layer
         * @param	text    text to display in it
         * @param	x_pct   how far across to put it,
         *                  0 = all the way to the left
         *                  1.0 = all the way to the right,
         * @param	y_pct   how far down to put it,
         *                  0 = at the top
         *                  1.0 = at the bottom
         */
        public function AddButtonCentered(text:String, callback:Function, x_pct:Number, y_pct:Number):void
        {
            var b:PulsingTextButton = _AddButton(text, callback);
            b.CenterOn( mArea.left + (mArea.width * x_pct),
                        mArea.top  + (mArea.height * y_pct) );
        }
    }
}
