﻿package meyermike.util
{
    import flash.geom.Point;
    import flash.geom.Rectangle;
    import meyermike.collision.CollisionInfo;
    import meyermike.constants.Defaults;
    import meyermike.core.GameObject;
    import meyermike.PlatformerCharacter;
    import meyermike.Util;
    import meyermike.util.SceneLayers;

    /**
     * ...
     * @author Mike Meyer
     */
    public class PhysicsUtil
    {
        public var Gravity:Number = Defaults.GRAVITY;
        public var MaxFallSpeed:Number = 600;

        private var TimeDelta:Number;

        /**
         * Run a physics step of the simulation
         */
        public function Update(layers:SceneLayers, timeDelta:Number):void
        {
            TimeDelta = timeDelta;
            ApplyGravity(layers);
            Collisions(layers);
        }

        /**
         * detect and react to collisions
         */
        protected function Collisions(layers:SceneLayers):void
        {
            layers.PlayerLayer.ForEach(CallPreCollisionCheck);
            layers.NpcLayer.ForEach(CallPreCollisionCheck);
            layers.EnemyLayer.ForEach(CallPreCollisionCheck);
            layers.ItemLayer.ForEach(CallPreCollisionCheck);

            layers.ForEachCartesianProductStyle(layers.PlayerLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.NpcLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.EnemyLayer, layers.PlatformLayer, CharacterPlatformCollision);
            layers.ForEachCartesianProductStyle(layers.ItemLayer, layers.PlatformLayer, CharacterPlatformCollision);

            layers.PlayerLayer.ForEach(CallPostCollisionCheck);
            layers.NpcLayer.ForEach(CallPostCollisionCheck);
            layers.EnemyLayer.ForEach(CallPostCollisionCheck);
            layers.ItemLayer.ForEach(CallPostCollisionCheck);
        }

        /**
         */
        protected function CallPreCollisionCheck(character:PlatformerCharacter):void
        {
            character.PreCollisionCheck(TimeDelta);
        }
        /**
         */
        protected function CallPostCollisionCheck(character:PlatformerCharacter):void
        {
            character.PostCollisionCheck(TimeDelta);
        }

        /**
         * detect and react to the player colliding with platforms
         */
        protected function CharacterPlatformCollision(character:PlatformerCharacter, platform:GameObject):void
        {
            if (character.PhysicsEnabled && platform.PhysicsEnabled)
            {
                var ci:CollisionInfo = Util.CheckAabbCollision(character, platform, TimeDelta);

                if (null != ci)
                {
                    var converted:CollisionInfo = ci.GlobalToLocal(character.parent);

                    // let the character know he hit something
                    character.CollisionOccurred(converted, platform);
                }
            }
        }

        /**
         * everything that can fall needs to have gravity applied
         */
        protected function ApplyGravity(layers:SceneLayers):void
        {
            layers.NpcLayer.ForEach(ApplyGravityTo);
            layers.PlayerLayer.ForEach(ApplyGravityTo);
            layers.EnemyLayer.ForEach(ApplyGravityTo);
            layers.BloodLayer.ForEach(ApplyGravityTo);
            layers.ItemLayer.ForEach(ApplyGravityTo);
        }

        /**
         * apply gravity to the object
         */
        protected function ApplyGravityTo(go:GameObject):void
        {
            if (go.PhysicsEnabled)
            {
                go.dy += Gravity * TimeDelta;

                if (go.dy > MaxFallSpeed)
                {
                    go.dy = MaxFallSpeed;
                }
            }
        }
    }

}
