﻿package meyermike.util
{
    import flash.display.DisplayObjectContainer;
    import flash.display.Sprite;
    import meyermike.core.GameObject;
    import meyermike.core.GameObjectHolder;

    /**
     * ...
     * @author Mike Meyer
     */
    public class SceneLayers
    {
        private var _backgroundLayer:GameObjectHolder;
        public function get BackgroundLayer():GameObjectHolder { return _backgroundLayer; }

        private var _enemyLayer:GameObjectHolder;
        public function get EnemyLayer():GameObjectHolder { return _enemyLayer; }

        private var _platformLayer:GameObjectHolder;
        public function get PlatformLayer():GameObjectHolder { return _platformLayer; }

        private var _playerLayer:GameObjectHolder;
        public function get PlayerLayer():GameObjectHolder { return _playerLayer; }

        private var _itemLayer:GameObjectHolder;
        public function get ItemLayer():GameObjectHolder { return _itemLayer; }

        private var _uiLayer:GameObjectHolder;
        public function get UiLayer():GameObjectHolder { return _uiLayer; }

        private var _npcLayer:GameObjectHolder;
        public function get NpcLayer():GameObjectHolder { return _npcLayer; }

        private var _bloodLayer:GameObjectHolder;
        public function get BloodLayer():GameObjectHolder { return _bloodLayer; }

        private var mLayers:/*GameObjectHolder*/Array;

        /**
         * call beat on all the layers
         */
        public function Beat(beat:Number):void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.Beat(beat);
            }
        }

        /**
         * call beat on all the layers
         */
        public function OffBeat(beat:Number):void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.OffBeat(beat);
            }
        }

        /**
         * call QuarterBeat on each layer
         */
        public function QuarterBeat(quarterbeat:Number):void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.QuarterBeat(quarterbeat);
            }
        }

        /**
         * call RemoveAll on each layer
         */
        public function RemoveAll():void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.RemoveAll();
            }
        }

        /**
         * call Update on each layer
         */
        public function Update(timeDelta:Number):void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.Update(timeDelta);
            }
        }

        /**
         * call Integrate on each layer
         */
        public function Integrate(timeDelta:Number):void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.Integrate(timeDelta);
            }
        }

        /**
         * call after all the final positions have been figured
         * just before drawing.
         * Calls PreDraw on each layer.
         */
        public function PreDraw():void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.PreDraw();
            }
        }

        /**
         * needs to be called once so that we can attach
         * the layers to the right container
         */
        public function Init(container:DisplayObjectContainer):void
        {
            _backgroundLayer = AddLayer(container);
            _enemyLayer = AddLayer(container);
            _platformLayer = AddLayer(container);
            _npcLayer = AddLayer(container);
            _playerLayer = AddLayer(container);
            _itemLayer = AddLayer(container);
            _bloodLayer = AddLayer(container);
            _uiLayer = AddLayer(container);
        }


        /**
         * create a layer and add it to the scene graph and layer list
         */
        private function AddLayer(container:DisplayObjectContainer):GameObjectHolder
        {
            if (null == mLayers) { mLayers = new Array(); }

            var layer:DisplayObjectContainer = new Sprite();
            container.addChild(layer);
            var goh:GameObjectHolder = new GameObjectHolder(layer)
            mLayers.push(goh);
            return goh;
        }

        /**
         * NO TIME FOR GOOD FUNCTION NAMES
         * for each thing in layer A, for each thing in layer B if call func like so
         *   func(thingFromLayerA, thingFromLayerB);
         */
        public function ForEachCartesianProductStyle(a:GameObjectHolder, b:GameObjectHolder, func:Function):void
        {
            for (var a_index:Number = 0; a_index < a.Objects.length; ++a_index)
            {
                var a_go:GameObject = a.Objects[a_index];

                for (var b_index:Number = 0; b_index < b.Objects.length; ++b_index)
                {
                    var b_go:GameObject = b.Objects[b_index];

                    func(a_go, b_go);
                }
            }
        }

        /**
         * clean up anything that htis could possibly leave dangling
         */
        public function Shutdown():void
        {
            for (var i:Number = 0; i < mLayers.length; ++i)
            {
                var layer:GameObjectHolder = mLayers[i];
                layer.RemoveAll();
                layer.Container.parent.removeChild(layer.Container);
            }

            mLayers = null;
        }
    }
}
