package meyermike.util
{
    import adobe.utils.CustomActions;
    import meyermike.core.GameObjectHolder;
    import meyermike.script.Step;
    import meyermike.script.step.Condition;
    import meyermike.script.step.CreateGameObjectFromBitmap;
    import meyermike.script.step.Loop;
    import meyermike.script.step.RunFunc;
    import meyermike.script.step.SpawnGameObject;
    import meyermike.script.step.Wait;

    /**
     * Utility for putting together scripts easily
     * @author Mike Meyer
     */
    public class ScriptBuilder
    {
        private var mScript:/*meyermike.script.Step*/Array;

        /**
         * get rid of any script still in progress and start a new one
         */
        public function BeginScript():void
        {
            mScript = [];
        }

        /**
         * finish the current script
         */
        public function EndScript():Step
        {
            var temp:/*meyermike.script.Step*/Array = mScript;
            mScript = null;
            return new Loop(1, temp);
        }

        /**
         * Add a step to block until a condition is met
         * @param	f
         */
        public function AddCondition(f:Function):void
        {
            if (null != mScript)
            {
                mScript.push(new Condition(f));
            }
        }

        /**
         * Add a GameObject spawn to the script
         * @param	layer        layer the object should appear on
         * @param	bitmapClass  image to create it from
         * @param	centerX      x coord to center it on
         * @param	centerY      y coord to center it on
         */
        public function AddSpawnFromBitmap(layer:GameObjectHolder, bitmapClass:Class, centerX:Number, centerY:Number):void
        {
            if (null != mScript)
            {
                mScript.push(new CreateGameObjectFromBitmap(layer, bitmapClass, centerX, centerY));
            }
        }

        /**
         * Add a function call to the script
         * @param	f  function to call
         */
        public function AddFunc(f:Function):void
        {
            if (null != mScript)
            {
                mScript.push(new RunFunc(f));
            }
        }

        /**
         * Add an GameObject spawn to the script
         * @param	layer      layer the object should appear on
         * @param	enemyType  GameObject class
         * @param	howMany    how many copies you want
         */
        public function AddObjectSpawn(layer:GameObjectHolder, enemyType:Class, howMany:Number = 1):void
        {
            if (null != mScript)
            {
                mScript.push(new SpawnGameObject(layer, enemyType, howMany));
            }
        }

        /**
         * Add a wait to the script
         * @param	numBeats how many beats to wait
         */
        public function AddWait(numBeats:Number = 1):void
        {
            if (null != mScript)
            {
                mScript.push(new Wait(numBeats));
            }
        }
    }
}
