﻿package meyermike.util.music
{
    import flash.errors.IOError;
    import flash.events.Event;
    import flash.events.EventDispatcher;
    import flash.events.IOErrorEvent;
    import flash.media.Sound;
    import flash.media.SoundChannel;
    import flash.media.SoundLoaderContext;
    import flash.media.SoundTransform;
    import flash.net.URLRequest;
    import flash.utils.Timer;
    import meyermike.util.ArrayUtil;
    import meyermike.util.music.MusicEvent;
    import meyermike.util.music.Song;
    import meyermike.Util;

    /**
     * Class to handle randomizing, looping, and streaming a playlist
     *
     * Example use:
     *
     *      var music:MusicUtil - new Music;
     *      music.Init( [ new Song( "http://8bitcollective.com/items/music/joule-girl_scrotum.mp3",
     *                              "Girl Scrotum",
     *                              "joule",
     *                              "http://8bitcollective.com/music/joule/girl+scrotum+%28OHC%29/"),
     *                    new Song( "http://8bitcollective.com/items/music/Untill%20I%20Sleep.mp3",
     *                              "Until I Sleep",
     *                              "henryhomesweet",
     *                              "http://8bitcollective.com/music/henryhomesweet/Until+I+Sleep/") ] );
     *      music.StartNewMusic();
     *
     * @author Mike Meyer
     */
    public class MusicUtil extends EventDispatcher
    {
        private var mMusic:Sound;
        private var mMusicChannel:SoundChannel;
        private var mPausePosition:Number;


        private var _currentSong:Song;
        /**
         * currently playing song.  may be null
         */
        public function get CurrentSong():Song { return _currentSong; }

        public function get IsPlaying():Boolean { return (null!=mMusicChannel); }

        private var _songList:Array;
        private var mInternalSongList:Array; // song list from which we have cull any broken links
        /**
         * List of songs that MusicUtil will play from
         */
        public function get SongList():Array { return _songList; }


        /**
         * List of songs that MusicUtil will play from
         */
        public function set SongList(value:Array):void
        {
            if (null != value)
            {
                _songList = value.concat();
                mInternalSongList = value.concat();
            }
            else
            {
                _songList = [];
                mInternalSongList = [];
            }
        }

        /**
         * Get the MusicUtil ready for use.  Note that if you don't provide
         * any songs here, you better do a .SongList = [...];
         * @param	playlist and array of meyermike.util.music.Songs
         */
        public function Init(playlist:/*meyermike.util.music.Song*/Array = null):void
        {
            SongList = playlist;
        }
        /**
         * Clean up anything this could possibly leave hanging
         */
        public function Shutdown():void
        {
            StopMusic();
            CleanUpSong();
        }

        /**
         * The song is going away or turned out to be invalid or something.
         * lose references to it
         */
        private function CleanUpSong():void
        {
            if (null != mMusic)
            {
                try
                {
                    mMusic.close();
                }
                catch (e:IOError)
                {
                }

                mMusic.removeEventListener(IOErrorEvent.IO_ERROR, SoundFail);
                mMusic.removeEventListener(Event.OPEN, SoundLoadBegin);
                mMusic = null;
            }

            if (null != _currentSong)
            {
                _currentSong = null;
            }
        }

        /**
         * Start looping a random song.
         */
        public function StartNewMusic():void
        {
            CleanUpSong();

            if (mInternalSongList.length > 0)
            {
                mMusic = new Sound();
                _currentSong = Util.RandomOneOf(mInternalSongList);
                var file:String = CurrentSong.url;
                var request:URLRequest = new URLRequest(file);
                var context:SoundLoaderContext = new SoundLoaderContext(8000, true);

                mMusic.addEventListener(IOErrorEvent.IO_ERROR, SoundFail);
                mMusic.addEventListener(Event.OPEN, SoundLoadBegin);
                mMusic.load(request, context);
            }
        }

        /**
         * music has been successfully found and can start playing
         */
        private function SoundLoadBegin(e:Event):void
        {
            dispatchEvent(new MusicEvent(MusicEvent.EVENT_NEW_SONG, CurrentSong));
            BeginPlaying();
        }

        /**
         *
         */
        private function SoundFail(e:IOErrorEvent):void
        {
            ArrayUtil.Remove(mInternalSongList, _currentSong);
            StartNewMusic();
        }

        /**
         * wrapper around StartNewMusic that can be used as an event listener
         * @param	e
         */
        private function _StartNewMusic(e:Event = null):void
        {
            StartNewMusic();
        }

        private function BeginPlaying(e:Event = null, position:Number = 0):void
        {
            if (null != mMusic)
            {
                StopMusic();

                mMusicChannel = mMusic.play(position, 1, new SoundTransform(0.5) );
                mMusicChannel.addEventListener(Event.SOUND_COMPLETE, _StartNewMusic);
            }
        }

        /**
         * Stop the current song
         */
        public function StopMusic():void
        {
            if (null != mMusicChannel)
            {
                mMusicChannel.removeEventListener(Event.SOUND_COMPLETE, _StartNewMusic);
                mMusicChannel.stop();
                mMusicChannel = null;
            }

            mPausePosition = 0;
        }

        /**
         * Pause the current song
         */
        public function Pause():void
        {
            if (null != mMusicChannel)
            {
                mPausePosition = mMusicChannel.position;
                mMusicChannel.stop();
                mMusicChannel = null;
            }
        }

        /**
         * Resume after pausing
         */
        public function Resume():void
        {
            BeginPlaying(null, mPausePosition);
        }
    }

}
