package meyermike.widget
{
    import flash.display.Sprite;
    import flash.events.MouseEvent;
    import flash.net.navigateToURL;
    import flash.net.URLRequest;
    import meyermike.constants.Colors;
    import meyermike.core.GameObject;
    import meyermike.script.Step;
    import meyermike.script.step.Animate;
    import meyermike.script.step.Loop;
    import meyermike.script.step.WaitTime;
    import meyermike.util.music.MusicEvent;
    import meyermike.util.music.MusicUtil;

    /**
     * Widget displaying music credits
     * @author Mike Meyer
     */
    public class MusicCredits extends GameObject
    {
        private const MUSIC_CREDITS_WIDTH:Number = 800;
        private const MUSIC_CREDITS_HEIGHT:Number = 45;
        private const MUSIC_CREDITS_MARGIN:Number = 5;
        private const MUSIC_CREDITS_INDENT:Number = 80;
        private const MUSIC_CREDITS_ROW_HEIGHT:Number = 11;
        private const LINK_COLOR:Number = 0x30C0FF;
        private const BACKGROUND_ALPHA:Number = 0.5;

        private const FADE_IN_TIME:Number = 0.5;
        private const FULL_ALPHA_TIME:Number = 5.0;
        private const FADE_OUT_TIME:Number = 1.0;

        private var mMusicUtil:MusicUtil;
        private var mBackground:Sprite;
        private var mMusicLabel:TextField;
        private var mMusicTitle:TextField;
        private var mMusicArtist:TextField;
        private var mMusicLink:TextField;
        private var mSongLink:String;

        private var mRunScript:Boolean = false;
        private var mAnimScript:Step;

        /**
         *
         */
        public function MusicCredits(musicUtil:MusicUtil)
        {
            mMusicUtil = musicUtil;
        }

        /**
         * this function called for both initial setup and to re-init for reuse
         */
        public override function Init():void
        {
            super.Init();

            CreateGraphics();

            mMusicUtil.addEventListener(MusicEvent.EVENT_NEW_SONG, SongChanged);

            mAnimScript = new Loop(1, [
                new Animate(this, { alpha:0 }, { alpha:1 }, FADE_IN_TIME),
                new WaitTime(FULL_ALPHA_TIME),
                new Animate(this, {alpha:1}, {alpha:0}, FADE_OUT_TIME) ] );

            // start out invisible
            this.visible = false;
        }

        /**
         * clean up
         */
        public override function Shutdown():void
        {
            super.Shutdown();
            mMusicUtil.removeEventListener(MusicEvent.EVENT_NEW_SONG, SongChanged);
        }

        /**
         * create the graphics for this widget
         */
        protected function CreateGraphics():void
        {
            if (null == mBackground)
            {
                mBackground = new Sprite();
                mBackground.graphics.beginFill( Colors.BLACK_ENOUGH );
                mBackground.graphics.drawRect(
                    0,
                    0,
                    MUSIC_CREDITS_WIDTH,
                    MUSIC_CREDITS_HEIGHT );
                mBackground.graphics.endFill();
                this.addChild(mBackground);

                mBackground.alpha = BACKGROUND_ALPHA;
            }

            if (null == mMusicLabel)
            {
                mMusicLabel = new TextField();
                mMusicLabel.x = MUSIC_CREDITS_MARGIN;
                mMusicLabel.y = MUSIC_CREDITS_MARGIN;
                mMusicLabel.SetFontSize(14);
                mMusicLabel.SetText("Music");
                this.addChild(mMusicLabel);
            }

            if (null == mMusicTitle)
            {
                mMusicTitle = new TextField();
                mMusicTitle.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                mMusicTitle.y = MUSIC_CREDITS_HEIGHT - ( (3*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                mMusicTitle.SetFontSize(12);
                mMusicTitle.SetText("");
                this.addChild(mMusicTitle);
            }

            if (null == mMusicArtist)
            {
                mMusicArtist = new TextField();
                mMusicArtist.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                mMusicArtist.y = MUSIC_CREDITS_HEIGHT - ( (2*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                mMusicArtist.SetFontSize(12);
                mMusicArtist.SetText("");
                this.addChild(mMusicArtist);
            }

            if (null == mMusicLink)
            {
                mMusicLink = new TextField();
                mMusicLink.x = MUSIC_CREDITS_MARGIN + MUSIC_CREDITS_INDENT;
                mMusicLink.y = MUSIC_CREDITS_HEIGHT - ( (1*MUSIC_CREDITS_ROW_HEIGHT) + MUSIC_CREDITS_MARGIN);
                mMusicLink.SetFontSize(12);
                mMusicLink.SetText("");
                this.addChild(mMusicLink);

                mMusicLink.SetFontColor( LINK_COLOR );
                mMusicLink.buttonMode = true;
                mMusicLink.mouseChildren = false;
                mMusicLink.addEventListener(MouseEvent.CLICK, SongLinkClicked);
            }
        }

        /**
         *
         */
        public override function Update(elapsedTime:Number):void
        {
            super.Update(elapsedTime);

            if (mRunScript)
            {
                var done:Boolean = mAnimScript.Execute(elapsedTime);

                if (done)
                {
                    mRunScript = false;
                    this.visible = false;
                }
            }
        }

        /**
         *
         */
        private function SongChanged(e:MusicEvent):void
        {
            mMusicTitle.SetText( e.song.title );
            mMusicArtist.SetText( "by " + e.song.artist );
            mMusicLink.SetText( "[link]" );
            mSongLink = e.song.link;

            mMusicLabel.alpha = 1;
            mMusicTitle.alpha = 1;
            mMusicArtist.alpha = 1;
            mMusicLink.alpha = 1;

            mRunScript = true;
            this.visible = true;
            mAnimScript.Restart();
        }

        /**
         * open up the song's page in a new window
         */
        private function SongLinkClicked(e:MouseEvent):void
        {
            navigateToURL(new URLRequest(mSongLink));
        }
    }
}
